package moncheck2;

import halloechen.JoalJFrame;
import halloechen.JoalJPanel;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.Border;
import java.util.ArrayList;
import java.util.concurrent.Executors; 
import java.util.concurrent.ExecutorService; 
import java.util.Date;

public class CarloUI implements ActionListener {

private JFrame myframe;
public JoalJPanel resultpanel;
private JButton thrstart;
private JButton thrstop;
private JButton reset;
private JButton buhelp;
private long seconds;		// Startzeit
private long statcyc;		// Aktuelle Zyklenzahl
private long oldstatcyc;	// Alte Zyklenzahl
private long sum = 0L;		// die aktuelle Summe der Lebenden
private long fixsum=0L;    // die Summe der schon Toten
private JTextField time;
private JTextField numtr;
private JTextField speed;
private JTextField cycles;
private ArrayList<CarloWorker> cw;
private ExecutorService exec;
private AbsResult simresult;
private AbsResult oldresult;
private AbsResult restype;
private CarloAbstract computer;
private CarloAggregator ca;


public CarloUI(String[] args)  {
	cw = new ArrayList<CarloWorker>();
	seconds = (new Date()).getTime();
	statcyc = 0L;
	oldstatcyc = 0L;
	
	String comp_class_name;
	if (args.length ==0 ) {
	comp_class_name = "moncheck2.CarloCompute";
	} else {
	comp_class_name = "moncheck2."+args[0];
	}
		
	try {
	Class compclass = Class.forName(comp_class_name,true,this.getClass().getClassLoader());
	computer = (CarloAbstract)compclass.newInstance();
	simresult= computer.getInstCoResult();	
	oldresult= computer.getInstCoResult();
	restype= computer.getInstCoResult();
	}
	catch (ClassNotFoundException cnf) {
    System.err.print("ClassNotFound!\nBitte geben Sie als Argument den Namen einer Tochterklasse von CarloAbstract an!\n");		
	}
	catch (InstantiationException ie) {
		System.err.println("Instantiation Exception");
	}
	catch (IllegalAccessException iae) {
		System.err.println("Illegal Access");
	}
	
	//System.out.println(simresult.toString());
	createAndShowGUI();
	CarloTimer ct = new CarloTimer(this);
	ct.execute();
	ca = new CarloAggregator(this);
	ca.execute();
	exec = Executors.newCachedThreadPool(); 
}

void createAndShowGUI() {
	myframe = new JFrame("Risiko-Würfeloptimierer");
	myframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	myframe.setLayout(new BorderLayout());
	Border border10 = BorderFactory.createEmptyBorder(10,10, 10, 10);
	JoalJPanel outpanel = new JoalJPanel();
	outpanel.getInP().setBorder(border10);
	JLabel labtime = new JLabel("Dauer",JLabel.CENTER);
	JLabel labcycles = new JLabel("Zyklen",JLabel.CENTER);
	JLabel labspeed = new JLabel("Geschwindigkeit",JLabel.CENTER);
	JLabel labnumtr = new JLabel("Anzahl Threads",JLabel.CENTER);
	GridLayout gridlay = new GridLayout(2,4);
	gridlay.setHgap(10);
	outpanel.getInP().setLayout(gridlay);	
	Border border = BorderFactory.createLoweredBevelBorder();
	time = new JTextField(15);
	cycles = new JTextField(15);
	speed = new JTextField(15);
	numtr = new JTextField(10);
    time.setEditable(false);
    time.setHorizontalAlignment(JTextField.RIGHT);
    time.setBorder(border);
    cycles.setEditable(false);
    cycles.setHorizontalAlignment(JTextField.RIGHT);
    cycles.setBorder(border);
    speed.setEditable(false);
    speed.setHorizontalAlignment(JTextField.RIGHT);
    speed.setBorder(border);
    numtr.setEditable(false);
    numtr.setBorder(border);
    numtr.setHorizontalAlignment(JTextField.RIGHT);
	outpanel.getInP().add(labtime);
    outpanel.getInP().add(labcycles);
    outpanel.getInP().add(labspeed);
    outpanel.getInP().add(labnumtr);
    outpanel.getInP().add(time);
	outpanel.getInP().add(cycles);
	outpanel.getInP().add(speed);
	outpanel.getInP().add(numtr);
	
	resultpanel = new JoalJPanel();
	computer.makeGuiPanel(resultpanel.getInP());
		
	JPanel butpanel = new JPanel();
	JButton ende = new JButton("Ende");
	ende.addActionListener(this);
	thrstart = new JButton("Thread starten");
	thrstart.addActionListener(new PlusThr(this,restype));
	thrstop = new JButton("Thread stoppen");
	thrstop.setEnabled(false);	
	thrstop.addActionListener(new MinusThr());
	reset = new JButton("Reset");
	reset.addActionListener(new ResetH(this));
	buhelp = new JButton("Hilfe");
	buhelp.addActionListener(new HilfeZ());
	butpanel.add(thrstart);
	butpanel.add(thrstop);
	butpanel.add(reset);
	butpanel.add(buhelp);
	butpanel.add(ende);
    myframe.add(outpanel,BorderLayout.PAGE_START);
    myframe.add(resultpanel,BorderLayout.CENTER);
    myframe.add(butpanel,BorderLayout.PAGE_END);
	myframe.pack();
	myframe.setVisible(true);
}

public void aggregate() {             //läuft im WorkerThreat!
	sum = 0L;
	simresult.reset();
	for (CarloWorker c: cw) {
		sum+=c.data.readPacket().cycles;
		simresult = add(simresult, c.data.readPacket().result);
	}
	statcyc = sum+fixsum;
	simresult = add(simresult, oldresult);
}

public AbsResult add(AbsResult su1, AbsResult su2) {
    return(computer.add(su1, su2));
}

public AbsResult compute(){
	return(computer.compute());
};

public void updateCycles() {         //läuft im EDT
	cycles.setText(Long.toString(statcyc));
	computer.updateDisplay(simresult);
	}

public void bringFinalResults (CarloPacket data) {
	fixsum+=data.cycles;
	oldresult = add(oldresult, data.result);
	CarloWorker.workercounter--;
	numtr.setText(Integer.toString(CarloWorker.workercounter));
	cw.remove(CarloWorker.workercounter);
	if (CarloWorker.workercounter == 0) {
		thrstop.setEnabled(false);
		reset.setEnabled(true);
		ca.cancel(true);
		System.out.print("Aggregator gecancelled \n");
	}
	System.out.print(cw.size()+" Objekte übrig.\n");
}
public void updateTiming() {
	time.setText(Long.toString(Math.round(Math.floor  (  ( (new Date()).getTime()-seconds) /1000) ) ));
	speed.setText(Long.toString(statcyc-oldstatcyc));
	oldstatcyc = statcyc;
}

public void actionPerformed(ActionEvent e) {
	System.exit(0);
}

class PlusThr implements ActionListener { 
	CarloUI cui;
	AbsResult res;
	public PlusThr(CarloUI _cui, AbsResult _res){
		cui = _cui;
		res = _res;
	}
	public void actionPerformed(ActionEvent aev) { 
		if (ca.isDone()) {
			ca = new CarloAggregator(cui);
			ca.execute();
		}
		computer.setParams();
		cw.add(new CarloWorker(cui,res));
		System.out.print("Es wurde CarloWorker Nr. "+(CarloWorker.workercounter-1)+" gestartet.\n");		
		exec.execute(cw.get(CarloWorker.workercounter-1));
		numtr.setText(Integer.toString(CarloWorker.workercounter));
		thrstop.setEnabled(true);
		reset.setEnabled(false);
		computer.enableUI(false);
	}
}

class MinusThr implements ActionListener { 
	public void actionPerformed(ActionEvent aev) { 
		cw.get(CarloWorker.workercounter-1).aufhoeren(true);
		
	}
}

class ResetH implements ActionListener { 
	CarloUI cui;
	public ResetH(CarloUI _cui){
		cui = _cui;
	}
	public void actionPerformed(ActionEvent aev) { 
		simresult = computer.reset();
		oldresult.reset();		
		computer.enableUI(true);
	}
}

class HilfeZ implements ActionListener { 
	public JoalJFrame myframe;
	
	public HilfeZ(){
	}
	public void actionPerformed(ActionEvent aev) { 
		
		myframe = new JoalJFrame("Hilfe");
		myframe.setDefaultCloseOperation(JoalJFrame.DISPOSE_ON_CLOSE);

		Container contentPane = myframe.getContentPane();
		JPanel pane1 = new JPanel();
		pane1.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		pane1.setLayout(new BorderLayout());
		contentPane.add(pane1);
		JTextArea txtfield = new JTextArea();
		txtfield.setMargin(new Insets(5, 5, 5, 5));
		txtfield.setLineWrap(true);
		txtfield.setWrapStyleWord(true);
		Dimension dim = new Dimension(550, 350);
		JScrollPane scrollPane = new JScrollPane(txtfield);
		scrollPane.setPreferredSize(dim);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		pane1.add(scrollPane, BorderLayout.CENTER);
		JButton buend = new JButton("Schließen");
		buend.addActionListener(new CloseHelp());
		JPanel pane2 = new JPanel();
		pane2.add(buend);
		pane1.add(pane2, BorderLayout.PAGE_END);
		txtfield.setText(computer.getHelpText());
		txtfield.setCaretPosition(0);
		myframe.pack();
		myframe.setVisible(true);
	}
	
	class CloseHelp implements ActionListener {
		public CloseHelp() {}
		public void actionPerformed(ActionEvent aev) {
		myframe.dispose();		
		}
		
	}
	
	
}


}